@@ -16,17 +16,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging, sys, os, re
-import gtk, gtk.glade
+from gi.repository import Gtk, Gdk
 
-import gettext
+#import gettext
 import locale
 
 GETTEXT_DOMAIN = 'autokey'
 
 locale.setlocale(locale.LC_ALL, '')
-for module in gtk.glade, gettext:
-    module.bindtextdomain(GETTEXT_DOMAIN)
-    module.textdomain(GETTEXT_DOMAIN)
+#for module in Gtk.glade, gettext:
+#    module.bindtextdomain(GETTEXT_DOMAIN)
+#    module.textdomain(GETTEXT_DOMAIN)
 
 
 __all__ = ["validate", "EMPTY_FIELD_REGEX", "AbbrSettingsDialog", "HotkeySettingsDialog", "WindowFilterSettingsDialog", "RecordDialog"]
@@ -45,8 +45,8 @@ EMPTY_FIELD_REGEX = re.compile(r"^ *$", re.UNICODE)
 
 def validate(expression, message, widget, parent):
     if not expression:
-        dlg = gtk.MessageDialog(parent, gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,
-                                 gtk.BUTTONS_OK, message)
+        dlg = Gtk.MessageDialog(parent, Gtk.DialogFlags.MODAL|Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.WARNING,
+                                 Gtk.ButtonsType.OK, message)
         dlg.run()
         dlg.destroy()
         if widget is not None:
@@ -62,16 +62,16 @@ class DialogBase:
 
     def on_close(self, widget, data=None):
         self.hide()
-        return gtk.TRUE    
+        return True    
 
     def on_cancel(self, widget, data=None):
         self.load(self.targetItem)
-        self.ui.response(gtk.RESPONSE_CANCEL)
+        self.ui.response(Gtk.ResponseType.CANCEL)
         self.hide()
         
     def on_ok(self, widget, data=None):
         if self.valid():
-            self.response(gtk.RESPONSE_OK)
+            self.response(Gtk.ResponseType.OK)
             self.hide()
         
     def __getattr__(self, attr):
@@ -110,21 +110,21 @@ class AbbrSettingsDialog(DialogBase):
         DialogBase.__init__(self)
         
         # set up list view
-        store = gtk.ListStore(str)
+        store = Gtk.ListStore(str)
         self.abbrList.set_model(store)
         
-        column1 = gtk.TreeViewColumn(_("Abbreviations"))
-        textRenderer = gtk.CellRendererText()
+        column1 = Gtk.TreeViewColumn(_("Abbreviations"))
+        textRenderer = Gtk.CellRendererText()
         textRenderer.set_property("editable", True)
         textRenderer.connect("edited", self.on_cell_modified)
         textRenderer.connect("editing-canceled", self.on_cell_editing_cancelled)
         column1.pack_end(textRenderer, True)
         column1.add_attribute(textRenderer, "text", 0)
-        column1.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+        column1.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
         self.abbrList.append_column(column1)
         
-        #for item in WORD_CHAR_OPTIONS_ORDERED:
-        #    self.wordCharCombo.append_text(item)
+        for item in WORD_CHAR_OPTIONS_ORDERED:
+            self.wordCharCombo.append_text(item)
         
     def load(self, item):
         self.targetItem = item
@@ -142,10 +142,19 @@ class AbbrSettingsDialog(DialogBase):
             
         self.removeTypedCheckbox.set_active(item.backspace)
         
-        for desc, regex in WORD_CHAR_OPTIONS.iteritems():
-            if item.get_word_chars() == regex:
-                self.wordCharCombo.set_active(WORD_CHAR_OPTIONS_ORDERED.index(desc))
-                break
+        self.__resetWordCharCombo()
+
+        wordCharRegex = item.get_word_chars()
+        if wordCharRegex in WORD_CHAR_OPTIONS.values():
+            # Default wordchar regex used
+            for desc, regex in WORD_CHAR_OPTIONS.iteritems():
+                if item.get_word_chars() == regex:
+                    self.wordCharCombo.set_active(WORD_CHAR_OPTIONS_ORDERED.index(desc))
+                    break
+        else:
+            # Custom wordchar regex used
+            self.wordCharCombo.append_text(model.extract_wordchars(wordCharRegex).encode("utf-8"))
+            self.wordCharCombo.set_active(len(WORD_CHAR_OPTIONS))
         
         if isinstance(item, model.Folder):
             self.omitTriggerCheckbox.hide()
@@ -171,7 +180,10 @@ class AbbrSettingsDialog(DialogBase):
         item.backspace = self.removeTypedCheckbox.get_active()
         
         option = self.wordCharCombo.get_active_text()
-        item.set_word_chars(WORD_CHAR_OPTIONS[option])
+        if option in WORD_CHAR_OPTIONS:
+            item.set_word_chars(WORD_CHAR_OPTIONS[option])
+        else:
+            item.set_word_chars(model.make_wordchar_re(option))
         
         if not isinstance(item, model.Folder):
             item.omitTrigger = self.omitTriggerCheckbox.get_active()
@@ -185,6 +197,7 @@ class AbbrSettingsDialog(DialogBase):
         
     def reset(self):
         self.abbrList.get_model().clear()
+        self.__resetWordCharCombo()
         self.removeButton.set_sensitive(False)
         self.wordCharCombo.set_active(0)
         self.omitTriggerCheckbox.set_active(False)
@@ -194,6 +207,12 @@ class AbbrSettingsDialog(DialogBase):
         self.triggerInsideCheckbox.set_active(False)
         self.immediateCheckbox.set_active(False)
         
+    def __resetWordCharCombo(self):
+        self.wordCharCombo.remove_all()
+        for item in WORD_CHAR_OPTIONS_ORDERED:
+            self.wordCharCombo.append_text(item)
+        self.wordCharCombo.set_active(0)
+        
     def get_abbrs(self):
         ret = []
         model = self.abbrList.get_model()
@@ -352,6 +371,7 @@ class HotkeySettingsDialog(DialogBase):
         self.setButton.set_sensitive(True)
             
     def set_key(self, key, modifiers=[]):
+        Gdk.threads_enter()
         if self.KEY_MAP.has_key(key):
             key = self.KEY_MAP[key]
         self._setKeyLabel(key)
@@ -363,10 +383,13 @@ class HotkeySettingsDialog(DialogBase):
         self.hyperButton.set_active(iomediator.Key.HYPER in modifiers)
         
         self.setButton.set_sensitive(True)
+        Gdk.threads_leave()
 
     def cancel_grab(self):
+        Gdk.threads_enter()
         self.setButton.set_sensitive(True)
         self._setKeyLabel(self.key)
+        Gdk.threads_leave()
         
     def build_modifiers(self):
         modifiers = []
@@ -440,7 +463,9 @@ class GlobalHotkeyDialog(HotkeySettingsDialog):
     def valid(self):
         configManager = self.configManager
         modifiers = self.build_modifiers()
-        pattern = self.targetItem.get_applicable_regex().pattern
+        regex = self.targetItem.get_applicable_regex()
+        pattern = None
+        if regex is not None: pattern = regex.pattern
 
         unique, conflicting = configManager.check_hotkey_unique(modifiers, self.key, pattern, self.targetItem)
         if not validate(unique, _("The hotkey is already in use for %s.") % conflicting, None,
@@ -454,14 +479,16 @@ class GlobalHotkeyDialog(HotkeySettingsDialog):
 
 class WindowFilterSettingsDialog(DialogBase):
     
-    def __init__(self, parent):
+    def __init__(self, parent, closure):
         builder = configwindow.get_ui("windowfiltersettings.xml")
         self.ui = builder.get_object("windowfiltersettings")
         builder.connect_signals(self)
         self.ui.set_transient_for(parent)
+        self.closure = closure
         
         self.triggerRegexEntry = builder.get_object("triggerRegexEntry")
         self.recursiveButton = builder.get_object("recursiveButton")
+        self.detectButton = builder.get_object("detectButton")
         
         DialogBase.__init__(self)
         
@@ -499,7 +526,65 @@ class WindowFilterSettingsDialog(DialogBase):
         return True
     
     def reset_focus(self):
-        self.triggerRegexEntry.grab_focus()        
+        self.triggerRegexEntry.grab_focus()
+        
+    def on_response(self, widget, responseId):
+        self.closure(responseId)
+        
+    def receive_window_info(self, info):
+        Gdk.threads_enter()
+        dlg = DetectDialog(self.ui)
+        dlg.populate(info)
+        response = dlg.run()
+        
+        if response == Gtk.ResponseType.OK:
+            self.triggerRegexEntry.set_text(dlg.get_choice().encode("utf-8"))
+            
+        self.detectButton.set_sensitive(True)
+        Gdk.threads_leave()
+        
+    def on_detectButton_pressed(self, widget, data=None):
+        #self.__dlg = 
+        widget.set_sensitive(False)
+        self.grabber = iomediator.WindowGrabber(self)
+        self.grabber.start()
+    
+        
+
+class DetectDialog(DialogBase):
+
+    def __init__(self, parent):
+        builder = configwindow.get_ui("detectdialog.xml")
+        self.ui = builder.get_object("detectdialog")
+        builder.connect_signals(self)
+        self.ui.set_transient_for(parent)
+        
+        self.classLabel = builder.get_object("classLabel")
+        self.titleLabel = builder.get_object("titleLabel")
+        self.classRadioButton = builder.get_object("classRadioButton")
+        self.titleRadioButton = builder.get_object("titleRadioButton")
+        
+        DialogBase.__init__(self)
+        
+    def populate(self, windowInfo):
+        self.titleLabel.set_text(_("Window title: %s") % windowInfo[0].encode("utf-8"))
+        self.classLabel.set_text(_("Window class: %s") % windowInfo[1].encode("utf-8"))
+        self.windowInfo = windowInfo
+
+    def get_choice(self):
+        if self.classRadioButton.get_active():
+            return self.windowInfo[1]
+        else:
+            return self.windowInfo[0]
+            
+    def on_cancel(self, widget, data=None):
+        self.ui.response(Gtk.ResponseType.CANCEL)
+        self.hide()
+        
+    def on_ok(self, widget, data=None):
+        self.response(Gtk.ResponseType.OK)
+        self.hide()
+
         
 class RecordDialog(DialogBase):
     
@@ -529,7 +614,7 @@ class RecordDialog(DialogBase):
         self.closure(responseId, self.get_record_keyboard(), self.get_record_mouse(), self.get_delay())
         
     def on_cancel(self, widget, data=None):
-        self.ui.response(gtk.RESPONSE_CANCEL)
+        self.ui.response(Gtk.ResponseType.CANCEL)
         self.hide()
             
     def valid(self):
