@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-__all__ = ["XRecordInterface", "EvDevInterface", "AtSpiInterface"]
+__all__ = ["XRecordInterface", "AtSpiInterface"]
 
 
 import os, threading, re, time, socket, select, logging, Queue, subprocess
@@ -39,7 +39,7 @@ import common
 if common.USING_QT:
     from PyQt4.QtGui import QClipboard, QApplication
 else:
-    import gtk
+    from gi.repository import Gtk, Gdk
 
 logger = logging.getLogger("interface")
 
@@ -82,8 +82,8 @@ class XInterfaceBase(threading.Thread):
         if common.USING_QT:
             self.clipBoard = QApplication.clipboard()
         else:
-            self.clipBoard = gtk.Clipboard()
-            self.selection = gtk.Clipboard(selection="PRIMARY")
+            self.clipBoard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+            self.selection = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
 
         self.__initMappings()
 
@@ -97,7 +97,7 @@ class XInterfaceBase(threading.Thread):
         self.__VisibleNameAtom = self.localDisplay.intern_atom("_NET_WM_VISIBLE_NAME", True)
         
         if not common.USING_QT:
-            self.keyMap = gtk.gdk.keymap_get_default()
+            self.keyMap = Gdk.Keymap.get_default()
             self.keyMap.connect("keys-changed", self.on_keys_changed)
         
         self.__ignoreRemap = False
@@ -219,7 +219,7 @@ class XInterfaceBase(threading.Thread):
             lines = output.splitlines()
             
             for line in lines:
-                if "gnome-shell" in line or "mutter" in line:
+                if "gnome-shell" in line or "cinnamon" in line:
                     return True
         except:
             pass # since this is just a nasty workaround, if anything goes wrong just disable it 
@@ -508,16 +508,21 @@ class XInterfaceBase(threading.Thread):
 
     def __restoreClipboard(self):
         if self.__savedClipboard != "":
-            self.clipBoard.setText(self.__savedClipboard, QClipboard.Clipboard)
+            if common.USING_QT:
+                self.clipBoard.setText(self.__savedClipboard, QClipboard.Clipboard)
+            else:
+                Gdk.threads_enter()
+                self.clipBoard.set_text(self.__savedClipboard)
+                Gdk.threads_leave()
 
     def __fillSelection(self, string):
         if common.USING_QT:
             self.clipBoard.setText(string, QClipboard.Selection)
             self.sem.release()
         else:
-            gtk.gdk.threads_enter()
+            Gdk.threads_enter()
             self.selection.set_text(string.encode("utf-8"))
-            gtk.gdk.threads_leave()
+            Gdk.threads_leave()
 
     def __fillClipboard(self, string):
         if common.USING_QT:
@@ -525,9 +530,12 @@ class XInterfaceBase(threading.Thread):
             self.clipBoard.setText(string, QClipboard.Clipboard)
             self.sem.release()
         else:
-            gtk.gdk.threads_enter()
+            Gdk.threads_enter()
+            text = self.clipBoard.wait_for_text()
+            self.__savedClipboard = ''
+            if text is not None: self.__savedClipboard = text
             self.clipBoard.set_text(string.encode("utf-8"))
-            gtk.gdk.threads_leave()
+            Gdk.threads_leave()
 
     def begin_send(self):
         self.__enqueue(self.__grab_keyboard)
@@ -861,7 +869,7 @@ class XInterfaceBase(threading.Thread):
 
     def __doQT4Workaround(self, keyCode):
         if len(self.lastChars) > 0:
-            if keyCode in self.lastChars and not self.lastChars[-1] == keyCode:
+            if keyCode in self.lastChars:
                 self.localDisplay.flush()
                 time.sleep(0.0125)
 
@@ -979,101 +987,6 @@ class XInterfaceBase(threading.Thread):
         self.join()
 
 
-class EvDevInterface(XInterfaceBase):
-
-    def initialise(self):
-        self.cancelling = False
-        self.connected = False
-        retryCount = 0
-
-        while not self.connected:
-            try:
-                self.__connect()
-            except:
-                if retryCount >= 6:
-                    raise
-
-            if self.cancelling:
-                break
-
-            if not self.connected:
-                retryCount += 1
-                time.sleep(10)
-
-
-    def cancel(self):
-        self.cancelling = True
-        XInterfaceBase.cancel(self)
-
-    def run(self):
-        logger.info("EvDev interface thread starting")
-        while True:
-            if self.cancelling:
-                logger.info("EvDev interface thread terminated")
-                break
-
-            try:
-                # Request next event
-                try:
-                    data = self.socket.recv(common.PACKET_SIZE)
-                except socket.timeout:
-                    continue # Timeout means no data to received
-
-
-                data = data.strip()
-                keyCode, button, state = data.split(',')
-
-
-                if keyCode:
-                    keyCode = int(keyCode)
-                    if state == '2':
-                        self.handle_keyrelease(keyCode)
-                        self.handle_keypress(keyCode)
-                    elif state == '1':
-                        self.handle_keypress(keyCode)
-                    elif state == '0':
-                        self.handle_keyrelease(keyCode)
-
-                if button:
-                    self.handle_mouseclick(button, None, None)
-
-            except:
-                logger.exception("Connection to EvDev daemon lost")
-                self.__connLost()
-                continue
-
-
-    def __connect(self):
-        # Connect to event daemon
-        logger.info("Attempting to establish connection to EvDev daemon")
-        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-        self.socket.settimeout(1)
-        try:
-            self.socket.connect(common.DOMAIN_SOCKET_PATH)
-            logger.info("EvDev daemon connected")
-            self.connected = True
-        except socket.error, e:
-            raise Exception("Unable to connect to EvDev daemon:\n" + str(e))
-
-    def __connLost(self):
-        # Called when the connection is lost - try to re-establish
-        # Loops every 10 seconds trying to establish a connection.
-        self.socket.close()
-        self.connected = False
-
-        while not self.connected:
-            try:
-                self.__connect()
-            except:
-                pass # don't care if something went wrong - keep retrying
-
-            if self.cancelling:
-                break
-
-            if not self.connected:
-                time.sleep(10)
-
-
 class XRecordInterface(XInterfaceBase):
 
     def initialise(self):
@@ -1157,7 +1070,7 @@ class AtSpiInterface(XInterfaceBase):
 
     def __processMouseEvent(self, event):
         if event.type[-1] == 'p':
-            button = int(event.type[-2])            
+            button = int(event.type[-2])
             self.handle_mouseclick(button, event.detail1, event.detail2)
 
     def __pumpEvents(self):
