@@ -20,7 +20,7 @@ import common, model, iomediator
 if common.USING_QT:
     from PyQt4.QtGui import QClipboard, QApplication
 else:
-    import gtk
+    from gi.repository import Gtk, Gdk
 
 class Keyboard:
     """
@@ -194,7 +194,33 @@ class Store(dict):
         Usage: C{store.remove_value(key)}
         """
         del self[key]
-
+        
+    def set_global_value(self, key, value):
+        """
+        Store a global value
+        
+        Usage: C{store.set_global_value(key, value)}
+        
+        The value stored with this method will be available to all scripts.
+        """
+        Store.GLOBALS[key] = value
+        
+    def get_global_value(self, key):
+        """
+        Get a global value
+        
+        Usage: C{store.get_global_value(key)}
+        """
+        return self.GLOBALS[key]        
+        
+    def remove_global_value(self, key):
+        """
+        Remove a global value
+        
+        Usage: C{store.remove_global_value(key)}
+        """
+        del self.GLOBALS[key]
+        
         
 class QtDialog:
     """
@@ -760,8 +786,8 @@ class GtkClipboard:
     """
     
     def __init__(self, app):
-        self.clipBoard = gtk.Clipboard()
-        self.selection = gtk.Clipboard(selection="PRIMARY")
+        self.clipBoard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        self.selection = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
         self.app = app
         
     def fill_selection(self, contents):
@@ -776,9 +802,9 @@ class GtkClipboard:
         self.__fillSelection(contents)
         
     def __fillSelection(self, string):
-        gtk.gdk.threads_enter()
+        Gdk.threads_enter()
         self.selection.set_text(string.encode("utf-8"))
-        gtk.gdk.threads_leave()
+        Gdk.threads_leave()
         #self.sem.release()
         
     def get_selection(self):
@@ -812,9 +838,9 @@ class GtkClipboard:
         self.__fillClipboard(contents)
         
     def __fillClipboard(self, string):
-        gtk.gdk.threads_enter()
+        Gdk.threads_enter()
         self.clipBoard.set_text(string.encode("utf-8"))
-        gtk.gdk.threads_leave()
+        Gdk.threads_leave()
         #self.sem.release()        
         
     def get_clipboard(self):
@@ -835,9 +861,9 @@ class GtkClipboard:
         
     def __execAsync(self, callback, *args):
         self.sem = threading.Semaphore(0)
-        gtk.gdk.threads_enter()
+        Gdk.threads_enter()
         callback(*args)
-        gtk.gdk.threads_leave()
+        Gdk.threads_leave()
         self.sem.acquire()
 
         
